// module HUDom2.java
// benutzt den DOM Parser zum Zeigen einer Struktur
// Mischt zwei DOM Baeume
// E.Huckert 05-2004

import javax.xml.parsers.*; 
import javax.xml.transform.*; 
import javax.xml.transform.dom.DOMSource; 
import javax.xml.transform.stream.StreamResult; 
import org.xml.sax.SAXException;  
import org.xml.sax.SAXParseException;  
import java.io.File;
import java.io.IOException;
import org.w3c.dom.*;

public class HUDom2
{
    // Global value so it can be ref'd by the tree-adapter
    static Document document1;
    static Document document2;
    static Node     inode = null;

    public HUDom2()
    {
    }      
    
    // druckt die Beschreibungsdaten des Knotens
    // wird auch missbraucht um einen Knoten (Unterbaum) zu importieren
    public void describeNode(Document doc,
                             Node node)
    {
      try
      {
        String name = node.getNodeName();
        System.out.println("Knotenname=" + name);
        if (name.equals("NNN"))
          // dieses Knoten importieren
          inode = document1.importNode(node, true);
        //name = ((Element)node).getTagName();
        //if (name != null)
        //  System.out.println("Tagname=" + name);
        if (node.getParentNode() == null)
          System.out.println("\tRoot!");
        short typ = node.getNodeType();
        System.out.println("\tType=" + typ);
        name = node.getNodeValue();
        if (name != null)
          System.out.println("\tValue=" + name);
        if (node.hasChildNodes())
        {
          NodeList nodes = node.getChildNodes();
          System.out.println("\tAnz. Kindknoten=" + nodes.getLength());
        }
        if (node.hasAttributes())
        {
           // Name und Wert der Attribute ermitteln
	   NamedNodeMap nm = node.getAttributes();
	   for (int i=0; i < nm.getLength(); i++)
           {
	      Node n = nm.item(i);
	      System.out.println("\t\tAttributename=" + n.getNodeName());
              name = n.getNodeValue();
              if (name != null)
                System.out.println("\t\tAttributwert=" + name);
              //Element element = (ELEMENT)n;
              //name = element.getAttribute(n.getNodeName());
              //System.out.println("\t\tAttributwert=" + name);
           }
        }
      }
      catch (Exception e)
      {
        System.out.println("Exception in describeNode():" + e.getMessage());
        e.printStackTrace();
      }
    }   // end describeNode()

    // Rekursiver Durchstieg durch den Baum       
    void recurse(Document doc, Node node)
    {
      NodeList nodes;
      Node child = null;
      //
      describeNode(doc, node);
      //
      nodes = node.getChildNodes();
      for (int i=0; i < nodes.getLength(); i++)
      {
        child = nodes.item(i);
        // rekursiver Aufruf
        recurse(doc, child);
      }
    }   // end recurse()

    public void followNodes(Document doc)
      throws IOException
    {
      Node root  = null;
      // DOCUMENT Knoten ausgeben
      String name = doc.getNodeName();
      System.out.println("Knotenname (Document)=" + name);
      // Root Knoten bestimmen
      root = doc.getDocumentElement();
      name = root.getNodeName();
      System.out.println("Wurzelknotenname=" + name);
      // Vorsicht: getElementTags() liefert nur die nicht-terminalen Knoten!
      //nodes = doc.getElementsByTagName("*");
      //
      // Die Rekursion anstossen
      recurse(doc, root);
    }   // end followNodes()

    // haengt den zuvor gewonnenen Teilbaum inode anstelle
    // des Teilbaums "BBB" in das aktuelle Dokument
    public void modifyTree(Document doc)
      throws IOException
    {
      Node child = null;
      NodeList nodes;
      nodes = doc.getElementsByTagName("*");
      String name = null;
      for (int i=0; i < nodes.getLength(); i++)
      {
        child = nodes.item(i);
        name = child.getNodeName();
        if (name.equals("BBB"))
        {
          try
          {
            (child.getParentNode()).replaceChild(inode, child);
            //child.appendChild(inode);
            break;
          }
          catch (Exception e)
          {
            System.out.println("Exception(modifyTree):" + e.getMessage());
          }
        }
      }
    }   // end modifyTree()

    // gibt ein Dokument auf System.out aus
    public void serialize(Document doc, String msg)
    {
      System.out.println("---------- " + msg + " -----------------");
      try
      {
        TransformerFactory tf = TransformerFactory.newInstance();
        Transformer t = tf.newTransformer();
        Source inp = new DOMSource(doc);
        Result outp = new StreamResult(System.out);
        t.transform(inp, outp);
      }
      catch (Exception e)
      {
        System.out.println("Exception(serialize): " + e.getMessage());
      }
      System.out.println("\n----------");
    }   // end serialize()

    public void run(String args[])
    {
        DocumentBuilderFactory factory =
            DocumentBuilderFactory.newInstance();
        factory.setValidating(true);   
        //factory.setNamespaceAware(true);
        factory.setNamespaceAware(true);
        try 
        {
           DocumentBuilder builder = factory.newDocumentBuilder();
           //
           // Dokument 1 parsen
           System.out.println("Dokument 1 wird geparst:" + args[0]);
           document1 = builder.parse(new File(args[0]));
           followNodes(document1);
           //
           // Dokument 2 parsen
           System.out.println("Dokument 2 wird geparst:" + args[1]);
           document2 = builder.parse(new File(args[1]));
           // 
           followNodes(document2);
           //
           // die beiden Baeume mischen
           // der Teilbaum inode wird in document1 eingefuegt (ersetzt
           // einen anderen Teilbaum)
           serialize(document1, "Baum 1");
           modifyTree(document1);
           //
           serialize(document2, "Baum 2");
           serialize(document1, "Baum 1");
           //
        }
        catch (SAXParseException spe) 
        {
           System.out.println("\n** Parsing error"
            + ", line " + spe.getLineNumber()
            + ", uri " + spe.getSystemId());
         System.out.println("   " + spe.getMessage() );

         Exception  x = spe;
         if (spe.getException() != null)
             x = spe.getException();
         x.printStackTrace();
      }
      catch (SAXException sxe) 
      {
         Exception  x = sxe;
         if (sxe.getException() != null)
             x = sxe.getException();
         x.printStackTrace();
      }
      catch (ParserConfigurationException pce) 
      {
          pce.printStackTrace();
      }
      catch (IOException ioe) 
      {
         ioe.printStackTrace();
      }
    }   // end run()

    public static void main(String args[])
    {
        if (args.length <  2) 
        {
          System.err.println("Usage: java HUDom2 filename1.xml filename2.xml");
          System.exit(1);
        }

        HUDom2 hudom2 = new HUDom2();
        hudom2.run(args);
    }    // main
}
