// module HUSax1.java
// Einfache Verwendung des SAX Parsers: soll sehr
// einfache HTML Dokumente aus XML erzeugen
// Dr.E.Huckert 05-2004

import org.xml.sax.*;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;
import java.io.*;

public class HUSax1 
       implements ContentHandler
{
  private Writer out;
  
  public HUSax1(Writer out) 
  {
    this.out = out;   
  }
    
  public void characters(char[] text, int start, int length)
      throws SAXException 
  {
    try 
    {
      out.write(text, start, length); 
    }
    catch (IOException e) 
    {
      throw new SAXException(e);   
    }
  }   // end characters() 
    
  public void startDocument()
      throws SAXException 
  {
    try 
    {
      out.write("<HTML>\n<BODY>\n"); 
    }
    catch (IOException e) 
    {
      throw new SAXException(e);   
    }
  }   // end startDocument()

  public void endDocument()
      throws SAXException 
  {
    try 
    {
      out.write("</BODY>\n</HTML>\n"); 
    }
    catch (IOException e) 
    {
      throw new SAXException(e);   
    }
  }  // end endDocument()
  
  public void startElement(String namespaceURI, 
                           String localName,
                           String qualifiedName, 
                           Attributes atts)
      throws SAXException 
  {
    try 
    {
      out.write("<BR><B>" + localName + ": </B>");
      // die Attribute und ihre Werte holen und ausgeben
      int ll = atts.getLength();
      for (int i=0; i < ll; i++)
      {
        out.write("&nbsp;<I>" + atts.getLocalName(i) + "=" +
                  atts.getValue(i) + "</I>&nbsp;");
      } 
    }
    catch (IOException e) 
    {
      throw new SAXException(e);   
    }
  }   // end startElement()
  
  public void endElement(String namespaceURI, 
                         String localName,
                         String qualifiedName) 
      throws SAXException 
  {
    try 
    { 
      if (localName.equals("BUCH"))
        out.write("<HR>\n");
    }
    catch (IOException e) 
    {
      throw new SAXException(e);   
    }
  }   // end endElement()
  
  
  // do-nothing methods
  public void setDocumentLocator(Locator locator) {}
  public void startPrefixMapping(String prefix, String uri) {}
  public void endPrefixMapping(String prefix) {}
  public void ignorableWhitespace(char[] text, int start, 
   int length) throws SAXException {}
  public void processingInstruction(String target, String data){}
  public void skippedEntity(String name) {}
 
  // DTDHandler
  public void notationDecl(String s, String publicId, String systemId)
  {
    System.out.println("DTDHandler: " + s);
  }

  public void unparsedEntityDecl(String s, String publicId, 
                                 String systemId, String notationName)
  {
    System.out.println("DTDHandler, unparsed: " + s);
  }

 

  public static void main(String[] args) 
  {
    if (args.length <= 0) 
    {
      System.out.println("Usage: java ExtractorDriver url");
      return;
    }
    // erspart uns "java -Dorg.xml.sax.driver=..."
    System.setProperty("org.xml.sax.driver",
                       "org.apache.crimson.parser.XMLReaderImpl"); 
    try 
    {
      XMLReader parser = XMLReaderFactory.createXMLReader();
      ((XMLReader)parser).setFeature("http://xml.org/sax/features/validation",
                                     true);
      ((XMLReader)parser).setFeature("http://xml.org/sax/features/namespaces",
                                     true);
      Writer out = new OutputStreamWriter(System.out);
      ContentHandler handler = new HUSax1(out);
      parser.setContentHandler(handler);
      //
      //DTDHandler dtdHandler = new DTDHandler();
      //parser.setDTDHandler(dtdHandler);
      //
      parser.parse(args[0]);
      out.flush();
    }
    catch (Exception e) 
    {
      System.err.println(e); 
    }
  } 
} // end HUSax1
